import os
import sys
import json
from subprocess import call

class Launcher:
    def __init__(self, dic=dict()):
        self.params = {
            "SOFA_PATH" : None,
            "G_MODE" : "batch",#qglviewer
            "SCENE_PATH" : None,
            "DT" : 0.01,
            "GRAVITY" : "0 0 0",
            "N_IT" : 100,
            "RESULT_DIR_PATH" : None
        }
        self.setParams(dic)

    def __getitem__(self, key):
        return self.params[key]

    def __setitem__(self, key, value):
        self.params[key] = value

    def checkRun(self):
        if self['SOFA_PATH'] is None:
            raise RuntimeError("Specify the path of sofa executable runSofa with SOFA_PATH")
        if self["SCENE_PATH"] is None:
            raise RuntimeError("Specify a path for the python sofa scene with SCENE_PATH")
        if self["RESULT_DIR_PATH"] is None:
            raise RuntimeError("Specify a prefix directory for the output with RESULT_DIR_PATH")
        if not os.path.exists(self["RESULT_DIR_PATH"]):
            os.mkdir(self["RESULT_DIR_PATH"])
        return True

    def __run__(self):
        if self.checkRun():
            jsonFileName = os.path.join(self.params["RESULT_DIR_PATH"], "params.json")
            string_launched = "{sofa_path} -g {g_mode} -n {it} {scene_path} --argv {argv}".format(
                    sofa_path=self.params["SOFA_PATH"],
                    g_mode=self.params["G_MODE"],
                    it=self.params["N_IT"],
                    scene_path=self.params["SCENE_PATH"],
                    argv=jsonFileName
            )

            json.dump(self.params, open(jsonFileName, 'w'))
            open(os.path.join(self.params["RESULT_DIR_PATH"], "string_launched.txt"),'w').write(string_launched)

            call(string_launched, shell=True)

    def run(self):
        # Function that can be overrode
        self.__run__()

    def setParams(self, dic):
        self.params = {**self.params, **dic}

    def setResDir(self, i=0):
        dirname = self.params["RESULT_DIR_PATH"]
        if dirname is None:
            raise RuntimeError("Specify a prefix directory for the output with RESULT_DIR_PATH")
        d = "{}_{}".format(dirname, i)
        if not os.path.exists(d):
            os.mkdir(d)
            self.params["RESULT_DIR_PATH"] = d
            return d
        else:
            return self.setResDir(i+1)
