import vtk
import os


class MeshReader:
    def __init__(self, filename):
        self.filename = filename

        if not os.path.exists(filename):
            raise FileNotFoundError(filename)

        reader = vtk.vtkUnstructuredGridReader()
        reader.SetFileName(filename)
        reader.Update()
        self.output = reader.GetOutput()

        self.positions = list()
        for i in range(self.output.GetNumberOfPoints()):
            self.positions.append(self.output.GetPoint(i))

        self.cells = self.output.GetCells()


    def __getattr__(self, att):
        s="\n"
        for k in dir(self):
            if "_" not in k and not "get" in k:
                s+=k+"\n"
        raise AttributeError("{} is no valid attribute\nHere is a list of the attributes : {}".format(att, s[:-1]))

    @property
    def vertices(self):
        return self.getCells(1)
    @property
    def lines(self):
        return self.getCells(3)
    @property
    def triangles(self):
        return self.getCells(5)
    @property
    def tetras(self):
        return self.getCells(10)
    @property
    def hexas(self):
        return self.getCells(12)

    @property
    def nbPoints(self):
        return len(self.positions)
    @property
    def nbCells(self):
        return self.output.GetNumberOfCells()
    @property
    def nbVertices(self):
        return len(self.vertices)
    @property
    def nbLines(self):
        return len(self.lines)
    @property
    def nbTriangles(self):
        return len(self.triangles)
    @property
    def nbTetras(self):
        return len(self.tetras)
    @property
    def nbHexas(self):
        return len(self.hexas)

    @property
    def cellsType(self):
        cT = list()
        for i in range(self.nbCells):
            cT.append(self.output.GetCellType(i))
        return cT

    def getCells(self, typ):
        cellsId = []
        if self.output is not None:
            for i in range(self.output.GetNumberOfCells()):
                if self.output.GetCellType(i)==typ:
                    cellsId.append(Cell(cellID=i, cellType=typ, cell=self.output.GetCell(i)))
        return cellsId



    def __str__(self):
        s = "\n   ** Class vtk unstructured grid reader **\n"
        s+= "Filename : {}\n".format(self.filename)
        s+= "Points : {}\n".format(self.nbPoints)
        s+= "Cells : {}\n".format(self.nbCells)
        s+= "Tetras : {}\n".format(self.nbTetras)
        s+= "Triangles : {}\n".format(self.nbTriangles)
        s+= "Vertices : {}\n".format(self.nbVertices)#if self.nbVertices>0:
        s+= "Lines : {}\n".format(self.nbLines)#if self.nbLines>0:
        s+= "Hexas : {}\n".format(self.nbHexas)#if self.nbHexas>0:
        return s+"\n"



class Cell:
    nbP = {1:1, 3:2, 5:3, 10:4, 12:8}
    ids = {1:"Vertex", 3:"Line", 5:"Triangle", 10:"Tetrahedron", 12:"Hexahedron"}

    def __init__(self, cellID, cellType, cell):
        self.type = cellType
        self.id = cellID
        pts = list()
        for i in range(Cell.nbP[cellType]):
            pts.append(cell.GetPointId(i))
        self.points = pts

    def __getitem__(self, i):
        return self.points[i]

    def __str__(self):
        s = "\n   ** Class Cell **\n"
        s += "Type : {} ({})\n".format(Cell.ids[self.type], self.type)
        s += "Id : {}\n".format(self.id)
        s += "Points : {}\n".format(" ".join(str(p) for p in self.points))
        return s











#
